from infinishell.infinibox.commands import set_renderer_query_fields, roles
from infinishell.infinibox.commands.infiniguard import ig_command, DDEActiveApp
from infinishell.infinibox.commands.infiniguard.lib.dde import DdeApi
from infinishell.infinibox.commands.infiniguard.lib.renderer import Fields, FieldDescr, renderfields
from infinishell.infinibox.commands.infiniguard.nodes import DdeAppCommands
from infinishell.infinibox.renderer import StorageCapacityRenderer
from infinishell.infishell import commandgroup, render

COMPACTION_FIELDS = Fields([
    FieldDescr('compaction_status', short_output=True),
    FieldDescr('status_progress', short_output=True),
    FieldDescr('start_time', short_output=True),
    FieldDescr('end_time', short_output=True),
    FieldDescr('compacted', short_output=True, renderer=StorageCapacityRenderer),
    FieldDescr('still_to_compact', short_output=True, renderer=StorageCapacityRenderer),
])

SUCCESS_MESSAGE = 'Command completed successfully'


@commandgroup(prefix="compaction", parent=DdeAppCommands)
class CompactionCommands(object):
    URL_PREFIX = 'api/compaction/'

    @ig_command(
        description="Start compaction",
        types=(DDEActiveApp,),
    )
    @roles.INFINIDAT.ADMIN
    def start(self, _api, dde):
        DdeApi(_api, dde.dde_instance_id).post(self.URL_PREFIX + 'start')
        return SUCCESS_MESSAGE

    @ig_command(
        description="Stop compaction",
        types=(DDEActiveApp,),
    )
    @roles.INFINIDAT.ADMIN
    def stop(self, _api, dde):
        DdeApi(_api, dde.dde_instance_id).post(self.URL_PREFIX + 'stop')
        return SUCCESS_MESSAGE

    @ig_command(
        description="View compaction status",
        types=(DDEActiveApp,),
    )
    @renderfields(
        value_renderers=COMPACTION_FIELDS.renderers,
        fields=COMPACTION_FIELDS.fieldnames,
        substitute=COMPACTION_FIELDS.substitute,
        detailed_fields=COMPACTION_FIELDS.detailedfieldnames,
        individually=True,
    )
    def status(self, _api, dde):
        set_renderer_query_fields()
        return DdeApi(_api, dde.dde_instance_id).get(self.URL_PREFIX + 'status')


RECLAMATION_FIELDS = Fields([
    FieldDescr('reclamation_status', short_output=True),
    FieldDescr('stage_status_progress', short_output=True),
    FieldDescr('total_progress', short_output=True),
    FieldDescr('start_time', short_output=True),
    FieldDescr('end_time', short_output=True),
])


@commandgroup(prefix="reclamation", parent=DdeAppCommands)
class ReclamationCommands(object):
    URL_PREFIX = 'api/reclamation/'

    @ig_command(
        description="Start reclamation",
        types=(DDEActiveApp,),
    )
    @roles.INFINIDAT.ADMIN
    def start(self, _api, dde):
        DdeApi(_api, dde.dde_instance_id).post(self.URL_PREFIX + 'start')
        return SUCCESS_MESSAGE

    @ig_command(
        description="Stop reclamation",
        types=(DDEActiveApp,),
    )
    @roles.INFINIDAT.ADMIN
    def stop(self, _api, dde):
        DdeApi(_api, dde.dde_instance_id).post(self.URL_PREFIX + 'stop')
        return SUCCESS_MESSAGE

    @ig_command(
        description="View reclamation status",
        types=(DDEActiveApp,),
    )
    @renderfields(
        value_renderers=RECLAMATION_FIELDS.renderers,
        fields=RECLAMATION_FIELDS.fieldnames,
        substitute=RECLAMATION_FIELDS.substitute,
        detailed_fields=RECLAMATION_FIELDS.detailedfieldnames,
        individually=True,
    )
    def status(self, _api, dde):
        return DdeApi(_api, dde.dde_instance_id).get(self.URL_PREFIX + 'status')
