from . import ig_command
from infinishell.infishell import commandgroup
from munch import Munch

from infinishell.infinibox.features import feature_status
from infinishell.infishell.utils.nothing import NOTHING
from infinishell.infinibox.commands.infiniguard.lib.dde import AnyDdeApi
from infinishell.infinibox.commands.options import OutputFormat
from infinishell.infinibox.commands.infiniguard.lib.renderer import render_multi_fields
from infinishell.infinibox.renderer.values import MillisecondsTimestampRenderer
from .lib.renderer import SuspendDeleteStateRenderer


INFO_ALLOCATED = "Capacity allocated for snapshots"
INFO_USED = "Capacity used by snapshots"
INFO_SNAPSHOTS = "Number of snapshots"
INFO_SUSPEND_EXPIRES_AT = "Snapshots deletion is suspended until"
INFO_SUSPEND_DELETE = "Snapshots Deletion"
CI_LONGEST_TITLE = max(len(INFO_SUSPEND_EXPIRES_AT), len(INFO_SUSPEND_DELETE),
                       len(INFO_ALLOCATED), len(INFO_USED), len(INFO_SNAPSHOTS))


def format_by_longest_line(line):
    return f"{line}{' ' * (CI_LONGEST_TITLE - len(line))}"


CI_SUBST = dict(
    allocated=format_by_longest_line(INFO_ALLOCATED),
    suspend_delete=format_by_longest_line(INFO_SUSPEND_DELETE),
    suspend_expires_at=format_by_longest_line(INFO_SUSPEND_EXPIRES_AT),
    used=format_by_longest_line(INFO_USED),
    snapshots=format_by_longest_line(INFO_SNAPSHOTS),
)

CI_FIELDS = list(CI_SUBST.keys())


def infinisafecommandgroup(cls=None, prefix=None, parent=None,
                           provides=(),
                           hidden=False,
                           manifest=NOTHING,
                           predicate=None):
    if predicate is None:
        predicate = feature_status("infinisafe", True)
    return commandgroup(cls, prefix, parent, provides,
                        hidden, manifest, predicate)


@infinisafecommandgroup(prefix="infinisafe")
class InfinisafeCommands(object):

    @ig_command(description="Display the InfiniSafe snapshots and capacity",
                options=[OutputFormat()])
    @render_multi_fields(
        value_renderers=dict(suspend_delete=SuspendDeleteStateRenderer(),
                             suspend_expires_at=MillisecondsTimestampRenderer()),
        fields=CI_FIELDS,
        detailed_individually=True,
        substitute=CI_SUBST,
        individually=True,
    )
    def info(self, _api):
        def build_app_info(apps, appid: str) -> Munch:
            app = apps[f'app_{appid}']
            d = Munch(used=app.allocated, snapshots=app.snapshots)
            sp = AnyDdeApi(_api).get(f'cr/policies/{appid}/suspend_delete')
            if sp.suspend_delete:
                d.suspend_expires_at = sp.expires_at
            else:
                d.suspend_delete = sp.suspend_delete
            return d

        report = AnyDdeApi(_api).get("cr/capacityreport")
        apps = report.ddeapps_capacity

        control = {
            "allow_json": True,
            "allow_csv": False,
            "additional_fields": [],
            "fields": CI_FIELDS
        }

        data = [
            Munch(
                data=[Munch(
                    allocated=report.capacity.provisioned,
                    used=report.capacity.allocated,
                    snapshots=report.capacity.snapshots)],
                title='Total',
                fields=CI_FIELDS),
            Munch(
                data=[build_app_info(apps, 'a')],
                title='DDE App A',
                fields=CI_FIELDS),
            Munch(
                data=[build_app_info(apps, 'b')],
                title='DDE App B',
                fields=CI_FIELDS)
        ]
        return {"control": control, "data": data}
