import time
from contextlib import contextmanager

from infinishell.infinibox.commands.infiniguard.lib.dde import DdeApi
from infinishell.infinibox.commands.utils import getch_pause
from infinishell.infishell import get_active_shell
from infinishell.utils import yesno

DEFAULT_CTRLBRK_CONFIRMATION = "You're going to abort the local command. " \
                               "The remote command will continue running on the server side until completion."


class AsyncTask:
    def __init__(self, _api, ddeid: int, result, keyboard_interrupt_warning: str = None):
        self.ddeapi = DdeApi(_api, ddeid)
        self.task_id = result['task_id']
        self.task_uri = f'api/asynctasks/{self.task_id}'
        self._done = False
        self._keyboard_interrupt_warning = keyboard_interrupt_warning

    @property
    def result(self):
        return self.ddeapi.get(self.task_uri)

    @property
    def done(self):
        if self._done:
            return True
        self._done = self.result['status'] != 'RUNNING'
        return self._done

    @contextmanager
    def keyboard_interrupt(self):
        try:
            yield
        except KeyboardInterrupt:
            if self._keyboard_interrupt_warning:
                get_active_shell().error(self._keyboard_interrupt_warning)
                if yesno():
                    raise
            else:
                raise

    def _iter_results(self, resource):
        mysleep = getch_pause if get_active_shell().is_interactive_terminal() else time.sleep

        index = 0
        resource_template = f'{self.task_uri}/{resource}' + \
                            '?sort=id&page_size=1000&page=1&id=gt:{}'

        with self.keyboard_interrupt():
            while not self.done:
                while result := self.ddeapi.get(resource_template.format(index)):
                    index = result[-1]['id']
                    for f in result:
                        yield f
                    mysleep(0)
                mysleep(1)

        for f in self.ddeapi.get(resource_template.format(index)):
            yield f

    @property
    def files(self):
        return self._iter_results('file_results')

    @property
    def lines(self):
        for line in self._iter_results('progress_results'):
            yield line['stdstream'], line['line']
