import ipaddress

from infinishell.infishell import BadValue
from infinishell.infishell.types import ArgType


class Password(ArgType):
    """ Simple string type """
    def get_description(self):
        return "password"

    def _get_actual(self, value):
        return True, value


class IPv4AddressNullable(ArgType):

    def _get_error(self):
        return "is not a valid IPv4 address"

    def get_description(self):
        return "A valid IPv4 address or empty string enclosed by double quotes"

    def verify_raw(self, arg, value):
        if value == 'none' or value == '':
            return
        try:
            ipaddress.IPv4Address(value)
        except ipaddress.AddressValueError:
            raise BadValue(self._get_error())
